<?php

/**
 * SMSComm SMS sending API client:
 *  - SMS, WapPush sending
 *  - SMS delivery reports
 *  - mass SMS packet sending
 *  - mass SMS delivery monitoring
 *  - schedule (delay) SMS for later date
 *
 * @package MBSLib
 * @author Mantas Litvaitis <ml@vero.lt>
 * @author Valdas Petrulis <vpe@vero.lt>
 *
 * @tutorial Simple connection example:
    <code>
    // Send SMS via API
    $client = new SMSComm('demo', 'demo');
    $result = $client->send('VeroDemo', '37065659515', 'Demo SMS API message');
    if ($result['status']=='success') {
        echo 'SMS sent';
    } else {
        echo 'Error in SMS: ' . $result['data']['field_error'];
    }
    </code>
 *
 * @tutorial Zend Framework connection example:
    <code>
    try {
        // Init SMS API client with wanted configuration
        $config = Zend_Registry::get('config');
        $client = new SMSComm($config->smscomm->username, $config->smscomm->password);
        //$client->setDebug(true);

        // Send SMS
        $result = $client->send('VeroDemo', '37065659515', 'Demo SMS API message');
        if ($result['status']=='success') {
            echo 'SMS sent';
        } else {
            echo 'Error in SMS: ' . $result['data']['field_error'];
        }
    } catch(Exception $e) {
        echo "Error in SMS API: " . $e;
    }
    </code>
 *
 */
class SMSComm
{
    /**
     * SMSComm username
     * @var string
     */
    private $_username;

    /**
     * SMSComm password
     * @var string
     */
    private $_password;

    /**
     * SMSComm API url for packet
     * @var string
     */
    private $_packetUrl = 'http://smscomm.vero.lt/partner/sms_packet.php';

    /**
     * SMSComm API url for single SMS
     * @var string
     */
    private $_smsUrl = 'http://smscomm.vero.lt/partner/sms.php';

    /**
     * Debug status
     * @var bool
     */
    private $_debug = false;

    /**
     * Debug type
     * @var string Possible values ('echo', 'buffer')
     */
    private $_debugType = 'echo';

    /**
     * Debug output buffer
     * @var string If debug type is 'buffer', then debug output is collected to this variable
     */
    private $_debugBuffer = null;

    /**
     * Object constructor
     *
     * @param string $username SMSComm username. Optional. Can be set later
     * @param string $password SMSComm password. Optional. Can be set later
     */
    function __construct($username=null, $password=null)
    {
        if (!empty($username) && !empty($password)) {
            $this->_username = $username;
            $this->_password = $password;
        }
    }

    /**
     * Sets SMSComm username
     *
     * @param string $username SMSComm username
     *
     * @throws SMSCommException
     */
    public function setUsername($username)
    {
        if (!empty($username)) {
            $this->_username = $username;
        } else {
            throw new SMSCommException('Empty SMSComm username');
        }
    }

    /**
     * Sets SMSComm password
     *
     * @param string $password SMSComm password
     *
     * @throws SMSCommException
     */
    public function setPassword($password)
    {
        if (!empty($password)) {
            $this->_password = $password;
        } else {
            throw new SMSCommException('Empty SMSComm password');
        }
    }

    /**
     * Sets SMSComm API url for packet sending
     *
     * @param string $url SMSComm API url
     *
     * @throws SMSCommException
     */
    public function setPacketUrl($url)
    {
        if (!empty($url)) {
            $this->_packetUrl = $url;
        } else {
            throw new SMSCommException('Empty SMSComm API packet url');
        }
    }

    /**
     * Sets SMSComm API url for single SMS sending
     *
     * @param string $url SMSComm API url
     *
     * @throws SMSCommException
     */
    public function setSmsUrl($url)
    {
        if (!empty($url)) {
            $this->_smsUrl = $url;
        } else {
            throw new SMSCommException('Empty SMSComm API SMS url');
        }
    }

    /**
     * Validate single message - check message for possible errors
     *
     * @param string $sender Sender phone number. This function is turned on by individual agreement, then all possible values should be listed
     * @param string $phone Receiver’s telephone number (MSISDN) in the international format "370xxxyyyyy"
     * @param string $message SMS text to be sent (in case of WapPush, the text is the link heading)
     * @param string $url (Optional) In case of WapPush, a link of WapPush to be sent
     * @param string $ident (Optional) External system unique identifier e.g. published advertisment unique number
     * @param string $type (Optional) Type of sending (SMS, WapPush), by default SMS
     * @param string $operator (Optional) To be indicated only if the partner knows the recipient’s operator. Format: tele2_lt, bite_lt, tele2_lv, etc.
     * @param string $sendDate (Optional) Message sending date. To be indicated only if the partner wants to send the message later. If not indicated, the message is sent immediately
     * @param string $dlrUrl (Optional) URL address, whereto the SMS delivery results should be sent
     * @param string $priority (Optional) Message priority. Higher priority is used if you need to send important messages (e.g. user registration approval) faster than other messages in queue (e.g. mass advertising)
     * @param string $charset (Optional) Sending text rule (charset). lisp – Lisped symbols, windows-1252 – Latin sybbols, windows-1257 – Baltic country symbols, windows-1251 – Cyrillic symbols, utf-8 – Any symbols
     * @param int $serviceId (Optional) MBS service ID, you will see seperated statictics of each service sending costs
     * @param int $partnerId (Optional) Partner ID
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return array
     *
     * On successful validation:
     * array(
     *   'status' => 'success',
     *   'data' => null,
     * )
     *
     * On validation error:
     * array(
     *  'status' => 'fail',
     *  'data' => array(
     *    'field_error' => 'Wrong msisdn format',
     *  )
     * )
     */
    public function validate($sender, $phone, $message, $url=null, $ident=null, $type='SMS', $operator=null, $sendDate=null, $dlrUrl=null, $priority=null, $charset=null, $serviceId=null, $partnerId=null)
    {
        $data = array(
            'ident' => $ident,
            'type' => $type,
            'sender' => $sender,
            'phone' => $phone,
            'sms' => $message,
            'send_date' => $sendDate,
            'operator' => $operator,
            'url' => $url,
            'charset' => $charset,
            'dlrurl' => $dlrUrl,
            'service_id' => $serviceId,
            'partner_id' => $partnerId,
            'priority' => $priority,
        );
        $result = $this->_callSmsAction('validate', $data);

        return $this->_parseSmsResponse($result);
    }

    /**
     * Send single message
     *
     * @param string $sender Sender phone number. This function is turned on by individual agreement, then all possible values should be listed
     * @param string $phone Receiver’s telephone number (MSISDN) in the international format "370xxxyyyyy"
     * @param string $message SMS text to be sent (in case of WapPush, the text is the link heading)
     * @param string $url (Optional) In case of WapPush, a link of WapPush to be sent
     * @param string $ident (Optional) External system unique identifier e.g. published advertisment unique number
     * @param string $type (Optional) Type of sending (SMS, WapPush), by default SMS
     * @param string $operator (Optional) To be indicated only if the partner knows the recipient’s operator. Format: tele2_lt, bite_lt, tele2_lv, etc.
     * @param string $sendDate (Optional) Message sending date. To be indicated only if the partner wants to send the message later. If not indicated, the message is sent immediately
     * @param string $dlrUrl (Optional) URL address, whereto the SMS delivery results should be sent
     * @param string $priority (Optional) Message priority. Higher priority is used if you need to send important messages (e.g. user registration approval) faster than other messages in queue (e.g. mass advertising)
     * @param string $charset (Optional) Sending text rule (charset). lisp – Lisped symbols, windows-1252 – Latin sybbols, windows-1257 – Baltic country symbols, windows-1251 – Cyrillic symbols, utf-8 – Any symbols
     * @param int $serviceId (Optional) MBS service ID, you will see seperated statictics of each service sending costs
     * @param int $partnerId (Optional) Partner ID
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return array
     *
     * On successful send:
     * array(
     *   'status' => 'success',
     *   'data' => array(
     *     'id' => '111'
     *   ),
     * )
     *
     * On validation error:
     * array(
     *   'status' => 'fail',
     *   'data' => array(
     *     'field_error' => 'Wrong msisdn format',
     *   )
     * )
     */
    public function send($sender, $phone, $message, $url=null, $ident=null, $type='SMS', $operator=null, $sendDate=null, $dlrUrl=null, $priority=null, $charset=null, $serviceId=null, $partnerId=null)
    {
        $data = array(
            'ident' => $ident,
            'type' => $type,
            'sender' => $sender,
            'phone' => $phone,
            'sms' => $message,
            'send_date' => $sendDate,
            'operator' => $operator,
            'url' => $url,
            'charset' => $charset,
            'dlrurl' => $dlrUrl,
            'service_id' => $serviceId,
            'partner_id' => $partnerId,
            'priority' => $priority,
        );
        $result = $this->_callSmsAction('send', $data);

        return $this->_parseSmsResponse($result);
    }

    /**
     * Dedicated to check mass message for possible errors, sending not performed.
     * All paramters are optional if they are described at each message in $messages array.
     * If for example $sender is set - it will be aplied to all $messages items, that does not have 'sender' set.
     *
     * @param array $messages Receiver's list - associative array, which contains key-value pairs. See examples below
     * @param string $sender Sender phone number. This function is turned on by individual agreement, then all possible values should be listed
     * @param string $phone Receiver’s telephone number (MSISDN) in the international format "370xxxyyyyy"
     * @param string $message SMS text to be sent (in case of WapPush, the text is the link heading)
     * @param string $url (Optional) In case of WapPush, a link of WapPush to be sent
     * @param string $ident (Optional) External system unique identifier e.g. mass message identifier
     * @param string $type (Optional) Type of sending (SMS, WapPush), by default SMS
     * @param string $operator (Optional) To be indicated only if the partner knows the recipient’s operator. Format: tele2_lt, bite_lt, tele2_lv, etc.
     * @param string $sendDate (Optional) Message sending date. To be indicated only if the partner wants to send the message later. If not indicated, the message is sent immediately
     * @param string $dlrUrl (Optional) URL address, whereto the SMS delivery results should be sent
     * @param string $priority (Optional) Message priority. Higher priority is used if you need to send important messages (e.g. user registration approval) faster than other messages in queue (e.g. mass advertising)
     * @param string $charset (Optional) Sending text rule (charset). lisp – Lisped symbols, windows-1252 – Latin sybbols, windows-1257 – Baltic country symbols, windows-1251 – Cyrillic symbols, utf-8 – Any symbols
     * @param int $serviceId (Optional) MBS service ID, you will see seperated statictics of each service sending costs
     * @param int $partnerId (Optional) Partner ID
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @example $messages can have different parameters sets:
     * A. Value must have required parameter "phone" - receiver's telephone number (MSISDN) in the international format "370xxxyyyyy":
     * $messages = array(
     *   array('phone' => '37065659515'),
     * );
     *
     * B. Key contains receiver unique identifier in list. Used later to relate with packet errors:
     * $messages = array(
     *   '5' => array('phone' => '37065659515'),
     *   'vero_1' => array('phone' => '37065659515'),
     * );
     *
     * C. Value can contain personal parameters of that receiver's message, e.g. by giving parameter "sms" different message text can be formed for specific receiver:
     * $messages = array(
     *   '5' => array(
     *     'phone' => '37065659515',
     *     'operator' => 'bite_lt',
     *  ),
     *  '6' => array(
     *    'phone' => '37065659515',
     *    'sms' => 'Different message for you',
     *    'send_date' => '2012-01-01 15:20:00'
     *  ),
     * );
     *
     * @return array
     * All given messages are valid and contains no errors:
     * array(
     *   'status' => 'success',
     *   'data' => array(
     *     'errors' => array()
     *   )
     * )
     *
     * Some of messages are not valid because of wrong data given:
     * array(
     *   'status' => 'fail',
     *   'data' => array(
     *     // rejected messages and their errors
     *     'errors' => array(
     *       '67' => array(
     *         'field' => 'phone', // which field contains error
     *         'field_error' => 'Wrong msisdn format', // error description
     *       )
     *     )
     *   )
     * )
     */
    public function validatePacket($messages, $sender=null, $message=null, $url=null, $ident=null, $type='SMS', $operator=null, $sendDate=null, $dlrUrl=null, $priority=null, $charset=null, $serviceId=null, $partnerId=null)
    {
        if (empty($messages)) {
            throw new SMSCommException('Messages can not be empty');
        }
        if (!is_array($messages)) {
            throw new SMSCommException('Messages must be an array');
        }

        $data = array(
            'ident' => $ident,
            'type' => $type,
            'sender' => $sender,
            'sms' => $message,
            'messages' => $messages,
            'send_date' => $sendDate,
            'operator' => $operator,
            'url' => $url,
            'charset' => $charset,
            'dlrurl' => $dlrUrl,
            'service_id' => $serviceId,
            'partner_id' => $partnerId,
            'priority' => $priority,
        );

        return $this->_callPacketAction('validate_packet', $data);
    }

    /**
     * Dedicated for mass message sending.
     * All paramters are optional if they are described at each message in $messages array.
     * If for example $sender is set - it will be aplied to all $messages items, that does not have 'sender' set.
     *
     * @param array $messages Receiver's list - associative array, which contains key-value pairs. See examples below
     * @param boolean $skipErrors (Optional) If TRUE, send messages in any case, errors will be skipped
     * @param string $sender Sender phone number. This function is turned on by individual agreement, then all possible values should be listed
     * @param string $phone Receiver’s telephone number (MSISDN) in the international format "370xxxyyyyy"
     * @param string $message SMS text to be sent (in case of WapPush, the text is the link heading)
     * @param string $url (Optional) In case of WapPush, a link of WapPush to be sent
     * @param string $ident (Optional) External system unique identifier e.g. mass message identifier
     * @param string $type (Optional) Type of sending (SMS, WapPush), by default SMS
     * @param string $operator (Optional) To be indicated only if the partner knows the recipient’s operator. Format: tele2_lt, bite_lt, tele2_lv, etc.
     * @param string $sendDate (Optional) Message sending date. To be indicated only if the partner wants to send the message later. If not indicated, the message is sent immediately
     * @param string $dlrUrl (Optional) URL address, whereto the SMS delivery results should be sent
     * @param string $priority (Optional) Message priority. Higher priority is used if you need to send important messages (e.g. user registration approval) faster than other messages in queue (e.g. mass advertising)
     * @param string $charset (Optional) Sending text rule (charset). lisp – Lisped symbols, windows-1252 – Latin sybbols, windows-1257 – Baltic country symbols, windows-1251 – Cyrillic symbols, utf-8 – Any symbols
     * @param int $serviceId (Optional) MBS service ID, you will see seperated statictics of each service sending costs
     * @param int $partnerId (Optional) Partner ID
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @example $messages can have different parameters sets:
     * A. Value must have required parameter "phone" - receiver's telephone number (MSISDN) in the international format "370xxxyyyyy":
     * $messages = array(
     *   array('phone' => '37065659515'),
     * );
     *
     * B. Key contains receiver unique identifier in list. Used later to relate with packet errors:
     * $messages = array(
     *   '5' => array('phone' => '37065659515'),
     *   'vero_1' => array('phone' => '37065659515'),
     * );
     *
     * C. Value can contain personal parameters of that receiver's message, e.g. by giving parameter "sms" different message text can be formed for specific receiver:
     * $messages = array(
     *   '5' => array(
     *     'phone' => '37065659515',
     *     'operator' => 'bite_lt',
     *  ),
     *  '6' => array(
     *    'phone' => '37065659515',
     *    'sms' => 'Different message for you',
     *    'send_date' => '2012-01-01 15:20:00'
     *  ),
     * );
     *
     * @return array
     * All messages or part of them successfully sent:
     * array(
     *   'status' => 'success',
     *   'data' => array(
     *     // succesfully accepted messages provided with a unique identifier in the MBS – SMS_ID
     *     'sent' => array(
     *       '66' => 123456,
     *     ),
     *     // rejected messages and their errors (then using parameter $skipErrors=true)
     *     'errors' => array(
     *       '67' => array(
     *         'field' => 'phone', // which field contains error
     *         'field_error' => 'Wrong msisdn format', // error description
     *       )
     *     )
     *  )
     * )
     *
     * Messages was not sent because of wrong data given:
     * array(
     *   'status' => 'fail',
     *   'data' => array(
     *     'sent' => array(),
     *     // rejected messages and their errors
     *     'errors' => array(
     *       '67' => array(
     *         'field' => 'phone', // which field contains error
     *         'field_error' => 'Wrong msisdn format', // error description
     *       )
     *     )
     *   )
     * )
     */
    public function sendPacket($messages, $skipErrors=false, $sender=null, $message=null, $url=null, $ident=null, $type='SMS', $operator=null, $sendDate=null, $dlrUrl=null, $priority=null, $charset=null, $serviceId=null, $partnerId=null)
    {
        if (empty($messages)) {
            throw new SMSCommException('Messages can not be empty');
        }
        if (!is_array($messages)) {
            throw new SMSCommException('Messages must be an array');
        }

        $data = array(
            'ident' => $ident,
            'type' => $type,
            'sender' => $sender,
            'sms' => $message,
            'messages' => $messages,
            'send_date' => $sendDate,
            'operator' => $operator,
            'url' => $url,
            'charset' => $charset,
            'dlrurl' => $dlrUrl,
            'service_id' => $serviceId,
            'partner_id' => $partnerId,
            'priority' => $priority,
            'skip_errors' => $skipErrors,
        );

        return $this->_callPacketAction('send_packet', $data);
    }

    /**
     * Dedicated to retrieve mass message sending, delivery statistics
     *
     * @param string $ident Indicate which messages to retrieve, by external system unique identifier e.g. mass message identifier
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return array Mass message information structure:
     * array(
     *   'ident' => 'news-reminder-12346', // external system identifier of messages
     *   'stats_overal' => 10, // amount of total messages
     *   'stats_sent' => 4, // amount of already sent messages
     *   'stats_delivered' => 2, // amount of messages delivered to users
     *   'stats_length' => 11, // amount of chunks to be paid (1 packet contains 160 characters)
     * )
     */
    public function getPacket($ident)
    {
        if (empty($ident)) {
            throw new SMSCommException('Ident is empty');
        }

        $data = array(
            'ident' => $ident
        );
        $result = $this->_callPacketAction('get_packet', $data);

        return $result['data'];
    }

    /**
     * Dedicated to edit unsent messages (one or overal packet).
     * At least one search parameter is required.
     * At least one update parameter is required.
     *
     * @param string $ident (Optional) Indicate which messages to edit, by external system unique identifier e.g. mass message identifier
     * @param int|array $messageId (Optional) Indicate which messages to edit, by  unique MBS  identifier or array of them
     * @param string $sender (Optional) If want to edit sender phone number
     * @param string $message (Optional) If want to edit SMS text to be sent (in case of WapPush, the text is the link heading)
     * @param string $url (Optional) If want to edit a link of WapPush to be sent
     * @param string $newIdent (Optional) If want to edit external system unique identifier e.g. edited messages can be formed to a new packet
     * @param string $type (Optional) If want to edit type of sending (SMS, WapPush)
     * @param string $sendDate (Optional) If want to edit message sending date
     * @param string $dlrUrl (Optional) If want to edit URL address, whereto the SMS delivery results should be sent
     * @param string $priority (Optional) If want to edit message priority
     * @param string $charset (Optional) If want to edit sending text rule (charset)
     * @param int $serviceId (Optional) If want to edit MBS service ID
     * @param int $partnerId (Optional) If want to edit partner ID
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return array
     * Messages successfully edited:
     * array(
     *   'status' => 'success',
     *   'data' => array(
     *     'edited' => 125, // amount of edited messages
     *   ),
     * )
     *
     * Messages was not edited because of wrong data data given:
     * array(
     *  'status' => 'fail',
     *  'data' => array(
     *    'field' => 'message', // which field contains error
     *    'field_error' => 'Empty sending text', // error description
     *  )
     * )
     */
    public function edit($ident=null, $messageId=null, $sender=null, $message=null, $url=null, $newIdent=null, $type = null, $sendDate=null, $dlrUrl=null, $priority=null, $charset=null, $serviceId=null, $partnerId=null)
    {
        if (empty($ident) && empty($messageId)) {
            throw new SMSCommException('Ident and message ID is empty. At least one must be filled');
        }

        $data = array(
            'ident' => $ident,
            'message_id' => $messageId,
            'type' => $type,
            'sender' => $sender,
            'sms' => $message,
            'send_date' => $sendDate,
            'url' => $url,
            'charset' => $charset,
            'dlrurl' => $dlrUrl,
            'service_id' => $serviceId,
            'partner_id' => $partnerId,
            'priority' => $priority,
            'new_ident' => $newIdent,
        );

        return $this->_callPacketAction('edit', $data);
    }

    /**
     * Dedicated to stop messages sending for unlimited time period (one or overal packet).
     * At least one search parameter is required.
     *
     * @param string $ident (Optional) Indicate which messages to stop, by external system unique identifier e.g. mass message identifier
     * @param int|array $messageId (Optional) Indicate which messages to stop, by unique MBS  identifier or array of them
     * @param string $newIdent (Optional) If want to edit external system unique identifier while stoping e.g. stopped messages can be formed to a new packet
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return int Ammount of stopped messages
     */
    public function stop($ident=null, $messageId=null, $newIdent=null)
    {
        if (empty($ident) && empty($messageId)) {
            throw new SMSCommException('Ident and message ID is empty. At least one must be filled');
        }

        $data = array(
            'ident' => $ident,
            'message_id' => $messageId,
            'new_ident' => $newIdent,
        );
        $result = $this->_callPacketAction('stop', $data);

        return $result['data']['edited'];
    }

    /**
     * Dedicated to unrecoverably remove unsent messages (one or overal packet).
     * At least one search parameter is required.
     *
     * @param string $ident (Optional) Indicate which messages to delete, by external system unique identifier e.g. mass message identifier
     * @param int|array $messageId (Optional) Indicate which messages to delete, by  unique MBS  identifier or array of them
     * @param string|array $phones (Optional) Phones to be deleted
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return int Amount of deleted messages
     */
    public function delete($ident=null, $messageId=null, $phones = null)
    {
        if (empty($ident) && empty($messageId) && empty($phones)) {
            throw new SMSCommException('Ident, message ID and phones are empty. At least one must be filled');
        }

        $data = array(
            'ident' => $ident,
            'message_id' => $messageId,
         	'phones' => $phones
        );
        $result = $this->_callPacketAction('delete', $data);

        return $result['data']['deleted'];
    }

    /**
     * Format data and call sms request
     *
     * @param string $action SMSComm API action
     * @param array $data Data to be transfered to SMSComm API
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return string Request result
     */
    protected function _callSmsAction($action, $data)
    {
        if (empty($action)) {
            throw new SMSCommException('No SmsComm API action specified');
        }

        $urlParams = array(
            'username' => $this->_username,
            'password' => $this->_password,
            'action' => $action,
        );

        $urlParams = array_merge($data, $urlParams);

        $result = $this->_sendRequest($this->_smsUrl, $urlParams);
        return $result;
    }

    /**
     * Format data and call packet request
     *
     * @param string $action SMSComm API action
     * @param array $data Data to be transfered to SMSComm API
     *
     * @throws SMSCommException
     * @throws SMSCommTransportException
     *
     * @return array Request result array
     */
    protected function _callPacketAction($action, $data)
    {
        if (empty($action)) {
            throw new SMSCommException('No SmsComm API action specified');
        }
        if (empty($data)) {
            throw new SMSCommException('No SmsComm API parameters specified');
        }

        $urlParams = array(
            'username' => $this->_username,
            'password' => $this->_password,
            'action' => $action,
        );

        $this->_debug('Calling action "'. $action .'" with params:');
        $this->_debug($data);
        $data = json_encode($data);
        if (json_last_error()) {
            throw new SMSCommException('An error occured when preparing parameters for request');
        }

        $result = $this->_sendRequest($this->_packetUrl, $urlParams, $data);

        // If decode failed - respons contains errors
        $result = json_decode($result, true);
        if (empty($result)) {
            throw new SMSCommTransportException('An error occured when parsing server response');
        }
        $this->_debug('Action "'. $action .'" returned params:');
        $this->_debug($result);

        // If fatal error returned - throw an exception
        if ($result['status'] == 'error') {
            throw new SMSCommException($result['message']);
        }
        return $result;
    }

    /**
     * Sends request to SMSComm API
     *
     * @param string $url SMSComm API url
     * @param array $urlParams GET parameters
     * @param string|array $postParams POST parameters
     *
     * @throws SMSCommTransportException
     *
     * @return array
     */
    protected function _sendRequest($url, $urlParams, $postParams=null)
    {
        if (empty($url)) {
            throw new SMSCommException('SMSComm API url is empty');
        }
        if (empty($urlParams)) {
            throw new SMSCommException('No paramters for SMSComm API specified');
        }

        // Prepare request url with aditional parameters
        $urlParams = $url . '?' . http_build_query($urlParams);
        $this->_debug('HTTP request to URL: '.$urlParams);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $urlParams);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        if (!empty($postParams)) {
            $this->_debug('HTTP POST params:');
            $this->_debug($postParams);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $postParams);
        }
        // Request can take a long time. Lets wait?
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 1200);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/plain'));

        $result = curl_exec($ch);
        // HTTP code check
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $errno = curl_errno($ch);
        curl_close($ch);

        $this->_debug('HTTP response code: '.$httpcode);
        $this->_debug('HTTP response: '.$result);

        // Request failed
        if ($httpcode != 200) {
            throw new SMSCommTransportException('An error occured connecting to SMSComm API. HTTP status: '.$httpcode.', curl error: '.$errno);
        // If request successful - return response
        } else {
            return $result;
        }
    }

    /**
     * Parse SMSComm API old style format for single sms and convert to array
     *
     * @param string $response SMSComm API SMS response
     *
     * @throws SMSCommTransportException
     *
     * @example
     *
     * $response = 'OK';
     * Returns:
     * array(
     *   'status' => 'success',
     *   'data' => null,
     * )
     *
     * $response = 'OK 95642';
     *
     * Returns
     * array(
     *   'status' => 'success',
     *   'data' => array(
     *     'id' => '95642',
     *   ),
     * )
     *
     * $response = 'ERROR Wrong MSISDN format';
     *
     * Returns:
     * array(
     *  'status' => 'fail',
     *  'data' => array(
     *    'field_error' => 'Wrong MSISDN format',
     *  )
     * )
     *
     * @return array
     */
    protected function _parseSmsResponse($response)
    {
        if (!preg_match('/^(ERROR|OK)\s?([\w\s]*)$/i', $response, $result)) {
            throw new SMSCommTransportException('An error occured when parsing server response');
        }

        if ($result[1] == 'OK') {
            return array(
                'status' => 'success',
                'data' => empty($result[2]) ? null : array('id' => $result[2]),
            );
        } else {
            return array(
                'status' => 'fail',
                'data' => array(
                    'field_error' => $result[2],
                ),
            );
        }

        return;
    }

    /**
     * Turns debug on/off
     * @param bools $state Debug state. True - for on, false for off
     */
    public function setDebug($state)
    {
        $this->_debug = $state;
    }

    /**
     * Returns debug status (on/off)
     * @return bool debug status
     */
    public function getDebug($state)
    {
        return $this->_debug;
    }

    /**
     * Sets debug type
     * @param string $type Debug type. Possible values: 'echo', 'buffer'
     *
     * @throws SMSCommException
     *
     * @example
     * if set to 'echo' - all debug output will be printed to screen
     *
     * if set to 'buffer' - all output goes to buffer.
     * use getDebugBuffer() to get buffer content
     */
    public function setDebugType($type)
    {
        if (in_array($type, array('echo', 'buffer'))) {
            $this->_debugType = $type;
        } else {
            throw new SMSCommException('Unsupported debug type: '.$type);
        }
    }

    /**
     * Get debug type
     *
     * @return string Possible values 'echo', 'buffer'
     */
    public function getDebugType()
    {
        return $this->_debugType;
    }

    /**
     * Get debug output buffer content
     *
     * @return string
     */
    public function getDebugBuffer()
    {
        return $this->_debugBuffer;
    }

    /**
     * Logs debug message depending to debug type
     *
     * @param mixed $message Debug message
     */
    private function _debug($message)
    {
        $debugOutput = null;

        if ($this->_debug) {
            if (is_string($message)) {
                $debugOutput = $message;
            } else {
                $debugOutput = '<pre>'.var_export($message, true).'</pre>';
            }
            $debugOutput .= '<br/>';

            if ($this->_debugType == 'echo') {
                echo $debugOutput;
            } else {
                $this->_debugBuffer .= $debugOutput;
            }
        }
    }
}

/**
 * Client general exception, used then API implementation not working, e.g.:
 *  - empty required paramaters
 *  - wrong type of parameters
 *  - fatal errors from SMSComm API
 *
 * @package MBSLib
 * @example SMSCommException('SMSComm API url is empty')
 */
class SMSCommException extends Exception
{
}

/**
 * Client transport exception, used for errors in communication with SMSComm API, e.g.:
 *  - HTTP errors
 *  - no internet connection
 *  - empty or invalid response
 *
 * @package MBSLib
 * @example SMSCommTransportException('An error occured connecting to SMSComm API')
 * @example SMSCommTransportException('An error occured when parsing server response')
 */
class SMSCommTransportException extends SMSCommException
{
}